##Data analysis of Kronholm et al. Marked Neurospora strains for competition experiments and Bayesian methods for fitness estimates

##Recommend reading this document with OrgStruct mode enabled, hook: ###

#Load HRM and qPCR functions
source("HRMqPCR.R")

#Load libraries for data manipulation and plotting
#if these are not installed need to install them using:
#install.packages("package_to_be_installed")
library(tidyr)
library(dplyr)
library(ggplot2)
library(cowplot)
library(colorspace)

##Note! Need to install Stan engine and package rstan first
##See installation instructions at:
##https://github.com/stan-dev/rstan/wiki/RStan-Getting-Started

##After Stan is installed install the rethinking package
#make sure that packages "coda", "mvtnorm" and "devtools" are installed
#library(devtools)
#devtools::install_github("rmcelreath/rethinking") #Install rethinking package

#For Bayesian analysis
library(rethinking)
options(mc.cores = parallel::detectCores())
library(tidybayes)

later:::ensureInitialized() #May need to run this

### * HRM assay development

hrmdat.RFU <- loadHrmDataFile("190326_hrm.csv") #Load RFU data, no need to modify the file manually

##Define active melt region (based on first derivate curve)
##For this data, c(75, 87)
hrmdat.RFU <- filter(hrmdat.RFU, Temperature >= 75 & Temperature <= 87)

##Normalization of RFU data
## I want to get RFU from 1 to 0, subtract minimum and then divide by the new maximum
##Normalize RFU data first
hrmdat.norm <- data.frame(Temperature = hrmdat.RFU[,1], apply(hrmdat.RFU[,-1], 2, RFUnorm))

#Find melting temperatures, using spline interpolation
hrm.melt <- findMeltingTemp(hrmdat.RFU, range = c(75, 87), step = 1e-4)

##Calculating data for a difference plot
##Normalizing for C1
hrmdat.dif <- hrmdat.norm[,-1] - hrmdat.norm$C1
hrmdat.dif <- data.frame(Temperature = hrmdat.RFU[,1], hrmdat.dif)
##Maximal difference in plot is for temperature:
hrmdat.dif[,1][which(hrmdat.dif$C11 == max(hrmdat.dif$C11))] #81.6

##Making a dataframe for standard curve
sample.summary <- data.frame(diff = as.numeric(hrmdat.dif[which(hrmdat.dif$C11 == max(hrmdat.dif$C11)),-1]), Tm = hrm.melt, prop = c(rep(1,10), 0, rep(0, 10), rep(seq(1,0,-0.1),2)), Sample = names(hrm.melt))

Bstd <- filter(sample.summary, grepl("C",sample.summary$Sample))

### ** Calculating standard curve


##Better to estimate standard curve using the model y = a + Bx, where x = proportion, that way we can avoid the wonky distribution of proportion
std.model <- map2stan(
    alist(
        diff ~ dnorm(mu, sigma),
        mu <- a + B*prop,
        a ~ dnorm(0,10), #Weakly regularizing priors
        B ~ dnorm(0,10),
        sigma ~ dcauchy(0,2)
        ),
    data = Bstd, chains = 2, cores = 2, warmup = 1000, iter = 3000)

##Extract posterior samples
post <- extract.samples(std.model)

##Model predictions
pred.data <- data.frame(prop = seq(from = -0.1, to = 1.1, by = 0.01))
std.pred <- link(std.model, pred.data)
pred.med <- apply(std.pred, 2, median)
pred.int <- apply(std.pred, 2, HPDI, prob = 0.95)

# simulate counterfactual outcomes
#R.sim <- sim(std.model, data=pred.data , n=1e4 )

#Converted std curve
pred.dif <- matrix(seq(from = -0.05, to = 0.40, by = 0.001), nrow = 1)
convr <- apply(pred.dif, 2, convert2prop, post = post)
conv.med <- apply(convr, 2, median)
conv.int <- apply(convr, 2, HPDI, prob = 0.95)

### *** Plotting (Figure 2)


##Data to long format
RFUdat <- gather(hrmdat.norm, key = "Sample", value = "RFU", -Temperature)
difdat <- gather(hrmdat.dif, key = "Sample", value = "diff", -Temperature)

#Since I have 11 levels I need to ramp up default Rcolor Brewer palette
#mypalette <- colorRampPalette(rev(brewer.pal(n = 9, name = "RdYlBu")))(11)

#From colorspace package
mypalette <- sequential_hcl(11, palette = "Viridis") #Viridis palette has good properties

#mypalette <-c("#A93154", "#B2396D", "#B84484", "#BC5198", "#BD60A9", "#BC6EB9", "#BA7EC5", "#B68CD0", "#B39BD9", "#AFA9E0", "#AEB6E5")

#Check the colours
pie(rep(1, length(mypalette)), col = mypalette, main = "")

##Normalized RFU

ggplot(RFUdat, aes(x = Temperature, y = RFU, group = Sample)) +
    geom_line()

Dil1 <- filter(RFUdat, grepl("A",RFUdat$Sample))
Dil2 <- filter(RFUdat, grepl("B",RFUdat$Sample))
Std1 <- filter(RFUdat, grepl("C",RFUdat$Sample))


#Recode function from dplyr package...
Std1$Sample <- recode(Std1$Sample, C1 = "1", C2 = "0.9", C3 = "0.8", C4 = "0.7", C5 = "0.6", C6 = "0.5", C7 = "0.4", C8 = "0.3", C9 = "0.2", C10 = "0.1", C11 = "0")

my.xlab <- expression(paste("Temperature (", degree,"C)"))
##Standard curve plot looks good
plot1 <- ggplot(Std1, aes(x = Temperature, y = RFU, group = Sample, colour = Sample)) +
    geom_line(size = 0.8) +
    scale_colour_manual(values = mypalette, name = "csr-1*\nproportion") +
    guides(color = guide_legend(override.aes = list(size = 2), keyheight = 0.5)) +
    xlab(label = my.xlab) +
    ylab("Normalized RFU") +
    theme( legend.position = c(0.7,0.6), legend.spacing.x = unit(0.05, "cm"))
    #guides(colour=guide_legend(keywidth=0.1))


###Difference plot

dif.Std1 <- filter(difdat, grepl("C",RFUdat$Sample))
dif.Std1$Sample <- recode(dif.Std1$Sample,  C1 = "1", C2 = "0.9", C3 = "0.8", C4 = "0.7", C5 = "0.6", C6 = "0.5", C7 = "0.4", C8 = "0.3", C9 = "0.2", C10 = "0.1", C11 = "0")


plot3 <- ggplot(dif.Std1, aes(x = Temperature, y = diff, group = Sample, colour = Sample)) +
    geom_line(size = 0.8) +
    #geom_vline(xintercept = 81.6, lty = "dashed") + #This looks good
    scale_colour_manual(values = mypalette) +
    xlab(label = my.xlab) +
    ylab("Difference (norm. RFU)") +
    theme(legend.position="none")

###First derivative of melting curve

f1 <- splinefun(hrmdat.norm$Temperature, hrmdat.norm$C1)
x1 <- seq(75, 87, by = 1e-2)
d1 <- -f1(x1, deriv = 1)

f2 <- splinefun(hrmdat.norm$Temperature, hrmdat.norm$C6)
d2 <- -f2(x1, deriv = 1)

f3 <- splinefun(hrmdat.norm$Temperature, hrmdat.norm$C11)
d3 <- -f3(x1, deriv = 1)

plot2 <- ggplot() +
    geom_line(aes(x = x1, y = d1), col = mypalette[11], size = 0.8) +
    geom_line(aes(x = x1, y = d2), col = mypalette[6], size = 0.8) +
    geom_line(aes(x = x1, y = d3), col = mypalette[1], size = 0.8) +
    xlab(label = my.xlab) +
    ylab(expression(paste(-d, " ", italic("norm. RFU") ,"/d", " ", italic(T)))) +
    geom_vline(xintercept = hrm.melt[22], lty = "dashed", col = mypalette[11], alpha = 0.5) +
    geom_vline(xintercept = hrm.melt[27], lty = "dashed", col = mypalette[6], alpha = 0.5) +
    geom_vline(xintercept = hrm.melt[32], lty = "dashed", col = mypalette[1], alpha = 0.5) +
    scale_x_continuous(limits=c(75,87.55), breaks = seq(75, 87.5, length.out = 6))
    
        

##Plotting the converted std curve
plot4 <- ggplot() +
    geom_ribbon(aes(ymin = conv.int[1,], ymax = conv.int[2,], x = as.vector(pred.dif)), fill = "grey70", alpha = 0.5) +
    #geom_ribbon(aes(ymin = conv.intpi[1,], ymax = conv.intpi[2,], x = as.vector(pred.dif)), fill = "grey70", alpha = 0.2) +    
    geom_line(aes(x = as.vector(pred.dif), y = conv.med)) +
    geom_point(aes(x = Bstd$diff, y = Bstd$prop)) +
    xlab("Difference (norm. RFU)") +
    ylab("csr-1* proportion") +
    scale_y_continuous(limits = c(0,1), breaks = seq(0, 1, 0.1))

plotgrid <- plot_grid(plot1, plot2, plot3, plot4, labels = c("A", "B", "C", "D"), align = "hv", ncol = 2)

save_plot("HRMplot.pdf", plotgrid, ncol = 2, nrow = 2, base_height=4, base_aspect_ratio = 1.3)


Cqdat <- loadHrmDataFile("~/Documents/tutkijatohtori/selection_exp/selection_epigenet/csr_tagging/HRM_results/190326/190326_hrmCq.csv")

##Assuming that sample.summary has already been loaded and calculated
sample.summary$Cq <- Cqdat$Cq

##Calculating the amount of conidia that were used for DNA extraction
##concentrations (conid / ml)
conc <- c(306656355.005, 100000000, 50000000, 10000000, 5000000, 1000000, 500000, 100000, 50000, 10000)
conc.ul <- conc/1000 #Concentration per microliter
final.conid <- conc.ul*40 #40 ul was used for DNA extraction

conc2 <- c(432961073.545, 100000000, 50000000, 10000000, 5000000, 1000000, 500000, 100000, 50000, 10000)
conc2.ul <- conc2/1000
final.conid2 <- conc2.ul*40

sample.summary$nconidia <- c(final.conid, 0, final.conid2, rep(4000000, 22))
sample.summary$logconidia <- log(sample.summary$nconidia, 10)

conidstd <- sample.summary[-11,]
conidstd <- conidstd[1:20,]
conidstd$standard <- factor(c(rep("std1", 10), rep("std2", 10)))

labels <- c(std1 = "csr-tag", std2 = "wild type csr")
pdf("Cq.pdf")
ggplot(conidstd, aes(x = logconidia, y = Cq, group = standard)) +
    geom_point() +
    geom_smooth(method = "lm") +
    xlab(label = expression(log[10](conidia))) +    
    facet_wrap(~standard, labeller = labeller(standard = labels))
dev.off()

summary(lm(Cq ~ logconidia + standard, data = conidstd))
summary(lm(Cq ~ logconidia + standard + logconidia:standard, data = conidstd)) #No effect of standard
summary(lm(Cq ~ logconidia, data = conidstd))


### * First competition experiment with csr-1* and mat locus

### ** Loading the data and calculating differences and melting curves

compexp.RFU <- loadHrmDataFile("190401_hrm.csv") #Load RFU data, no need to modify the file manually
compexp.info <- read.csv("190401_sampleinfo.csv", header = T) #Load sample information
compexp.Cq <- loadHrmDataFile("190401_Cq.csv")

##Define active melt region (based on first derivate curve)
##For this data, c(75, 87)
compexp.RFU <- filter(compexp.RFU, Temperature >= 75 & Temperature <= 87)

##Normalization of RFU data
## I want to get RFU from 1 to 0, subtract minimum and then divide by the new maximum
##Normalize RFU data first
compexp.norm <- data.frame(Temperature = compexp.RFU[,1], apply(compexp.RFU[,-1], 2, RFUnorm))

#Find melting temperatures, using spline interpolation
compexp.melt <- findMeltingTemp(compexp.norm, range = c(75, 87), step = 1e-4)

##Calculating data for a difference plot
##Normalizing for B12, which is pos. cont. for csr-tag
compexp.dif <- compexp.norm[,-1] - compexp.norm$B12
compexp.dif <- data.frame(Temperature = compexp.RFU[,1], compexp.dif)
##Maximal difference in plot is for temperature:
compexp.dif[,1][which(compexp.dif$B11 == max(compexp.dif$B11))] #81.4

##Making a combined dataframe
compexp.info$diff <- as.numeric(compexp.dif[which(compexp.dif$B11 == max(compexp.dif$B11)),-1])
compexp.info$Tm <- compexp.melt
compexp.info$Cq <- compexp.Cq$Cq

#sample.summary <- data.frame(diff = as.numeric(compexp.dif[which(compexp.dif$B11 == max(compexp.dif$B11)),-1]), Tm = compexp.melt, prop = c(rep(1,10), 0, rep(0, 10), rep(seq(1,0,-0.1),2)), Sample = names(compexp.melt))

Bstd <- filter(compexp.info, sampletype == "std")
Bstd <- Bstd[,c(1:3,12:14)]

#Bstd <- filter(sample.summary, grepl("C",sample.summary$Sample))

ggplot(Bstd, aes(x = diff, y = proportion)) +
    geom_smooth(method = "lm") +
    geom_point()

### *** Fitting the standard curve

##Better to estimate standard curve using the model y = a + Bx, where x = proportion, that way we can avoid the wonky distribution of proportion
std.model <- map2stan(
    alist(
        diff ~ dnorm(mu, sigma),
        mu <- a + B*proportion,
        a ~ dnorm(0,10), #Weakly regularizing priors
        B ~ dnorm(0,10),
        sigma ~ dcauchy(0,2)
        ),
    data = Bstd, chains = 2, cores = 2, warmup = 1000, iter = 3000)

##Extract posterior samples
post <- extract.samples(std.model)

#Converted std curve
pred.dif <- matrix(seq(from = -0.05, to = 0.45, by = 0.001), nrow = 1)
convr <- apply(pred.dif, 2, convert2prop, post = post)
conv.med <- apply(convr, 2, median)
conv.int <- apply(convr, 2, HPDI, prob = 0.95)

##plotting std curve
 ggplot() +
    geom_ribbon(aes(ymin = conv.int[1,], ymax = conv.int[2,], x = as.vector(pred.dif)), fill = "grey70", alpha = 0.5) +
    #geom_ribbon(aes(ymin = conv.intpi[1,], ymax = conv.intpi[2,], x = as.vector(pred.dif)), fill = "grey70", alpha = 0.2) +    
    geom_line(aes(x = as.vector(pred.dif), y = conv.med)) +
    geom_point(aes(x = Bstd$diff, y = Bstd$proportion)) +
    xlab("Difference (norm. RFU)") +
    ylab("csr-tag proportion") +
    scale_y_continuous(limits = c(0,1), breaks = seq(0, 1, 0.1))

##
##Proportions in unknown samples
compexp.sample <- filter(compexp.info, sampletype == "sample")

foo <- list(0)
results <- rep(foo,nrow(compexp.sample))
for(i in 1:length(results)) { results[[i]] <- convert2prop(post, compexp.sample[i,13]) }

compexp.sample$propest <- unlist(lapply(results, median))
prop.hpdi <- matrix(unlist(lapply(results, HPDI, prob = 0.95)), ncol = 2, byrow = T)
compexp.sample$proplow <- prop.hpdi[,1]
compexp.sample$prophigh <- prop.hpdi[,2]
compexp.sample$propsd <- unlist(lapply(results, sd))

### ** Loading data from the second trial, plate 1

compexpnew.RFU <- loadHrmDataFile("191212_1_hrm.csv") #Load RFU data, no need to modify the file manually
compexpnew.info <- read.csv("191212_1_sample_info.csv", header = T) #Load sample information
compexpnew.Cq <- loadHrmDataFile("191212_1_Cq.csv")

##Define active melt region (based on first derivate curve)
##For this data, c(75, 87)
compexpnew.RFU <- filter(compexpnew.RFU, Temperature >= 75 & Temperature <= 87)

##Normalization of RFU data
## I want to get RFU from 1 to 0, subtract minimum and then divide by the new maximum
##Normalize RFU data first
compexpnew.norm <- data.frame(Temperature = compexpnew.RFU[,1], apply(compexpnew.RFU[,-1], 2, RFUnorm))

#Find melting temperatures, using spline interpolation
compexpnew.melt <- findMeltingTemp(compexpnew.norm, range = c(75, 87), step = 1e-4)

##Calculating data for a difference plot
##Normalizing for B12, which is pos. cont. for csr-tag
compexpnew.dif <- compexpnew.norm[,-1] - compexpnew.norm$B12
compexpnew.dif <- data.frame(Temperature = compexpnew.RFU[,1], compexpnew.dif)
##Maximal difference in plot is for temperature:
compexpnew.dif[,1][which(compexpnew.dif$C12 == max(compexpnew.dif$C12))] #81.6

##Making a combined dataframe
compexpnew.info$diff <- as.numeric(compexpnew.dif[which(compexpnew.dif$C12 == max(compexpnew.dif$C12)),-1])
compexpnew.info$Tm <- compexpnew.melt
compexpnew.info$Cq <- compexpnew.Cq$Cq

Bstd <- filter(compexpnew.info, sampletype == "std")
Bstd <- Bstd[,c(1:3,13:15)]

### *** Fitting the standard curve for trial 2, plate 1

##Better to estimate standard curve using the model y = a + Bx, where x = proportion, that way we can avoid the wonky distribution of proportion
std.model <- map2stan(
    alist(
        diff ~ dnorm(mu, sigma),
        mu <- a + B*proportion,
        a ~ dnorm(0,10), #Weakly regularizing priors
        B ~ dnorm(0,10),
        sigma ~ dcauchy(0,2)
        ),
    data = Bstd, chains = 2, cores = 2, warmup = 1000, iter = 3000)

##Extract posterior samples
post <- extract.samples(std.model)

#Converted std curve
pred.dif <- matrix(seq(from = -0.05, to = 0.45, by = 0.001), nrow = 1)
convr <- apply(pred.dif, 2, convert2prop, post = post)
conv.med <- apply(convr, 2, median)
conv.int <- apply(convr, 2, HPDI, prob = 0.95)

##Proportions in unknown samples
compexpnew.sample <- filter(compexpnew.info, sampletype == "sample")

foo <- list(0)
results <- rep(foo,nrow(compexpnew.sample))
for(i in 1:length(results)) { results[[i]] <- convert2prop(post, compexpnew.sample[i,13]) }

compexpnew.sample$propest <- unlist(lapply(results, median))
prop.hpdi <- matrix(unlist(lapply(results, HPDI, prob = 0.95)), ncol = 2, byrow = T)
compexpnew.sample$proplow <- prop.hpdi[,1]
compexpnew.sample$prophigh <- prop.hpdi[,2]
compexpnew.sample$propsd <- unlist(lapply(results, sd))

### ** Loading data from second trial plate 2

compexpnew2.RFU <- loadHrmDataFile("191212_2_hrm.csv") #Load RFU data, no need to modify the file manually
compexpnew2.info <- read.csv("191212_2_sample_info.csv", header = T) #Load sample information
compexpnew2.Cq <- loadHrmDataFile("191212_2_Cq.csv")

##Define active melt region (based on first derivate curve)
##For this data, c(75, 87)
compexpnew2.RFU <- filter(compexpnew2.RFU, Temperature >= 75 & Temperature <= 87)

##Normalization of RFU data
## I want to get RFU from 1 to 0, subtract minimum and then divide by the new maximum
##Normalize RFU data first
compexpnew2.norm <- data.frame(Temperature = compexpnew2.RFU[,1], apply(compexpnew2.RFU[,-1], 2, RFUnorm))

#Find melting temperatures, using spline interpolation
compexpnew2.melt <- findMeltingTemp(compexpnew2.norm, range = c(75, 87), step = 1e-4)

##Calculating data for a difference plot
##Normalizing for B12, which is pos. cont. for csr-tag
compexpnew2.dif <- compexpnew2.norm[,-1] - compexpnew2.norm$B12
compexpnew2.dif <- data.frame(Temperature = compexpnew2.RFU[,1], compexpnew2.dif)
##Maximal difference in plot is for temperature:
compexpnew2.dif[,1][which(compexpnew2.dif$C12 == max(compexpnew2.dif$C12))] #81.6

##Making a combined dataframe
compexpnew2.info$diff <- as.numeric(compexpnew2.dif[which(compexpnew2.dif$C12 == max(compexpnew2.dif$C12)),-1])
compexpnew2.info$Tm <- compexpnew2.melt
compexpnew2.info$Cq <- compexpnew2.Cq$Cq

Bstd <- filter(compexpnew2.info, sampletype == "std")
Bstd <- Bstd[,c(1:3,13:15)]

### *** Fitting the standard curve for trial 2, plate 2

##Better to estimate standard curve using the model y = a + Bx, where x = proportion, that way we can avoid the wonky distribution of proportion
std.model <- map2stan(
    alist(
        diff ~ dnorm(mu, sigma),
        mu <- a + B*proportion,
        a ~ dnorm(0,10), #Weakly regularizing priors
        B ~ dnorm(0,10),
        sigma ~ dcauchy(0,2)
        ),
    data = Bstd, chains = 2, cores = 2, warmup = 1000, iter = 3000)

##Extract posterior samples
post <- extract.samples(std.model)

#Converted std curve
pred.dif <- matrix(seq(from = -0.05, to = 0.45, by = 0.001), nrow = 1)
convr <- apply(pred.dif, 2, convert2prop, post = post)
conv.med <- apply(convr, 2, median)
conv.int <- apply(convr, 2, HPDI, prob = 0.95)

##
##Proportions in unknown samples
compexpnew2.sample <- filter(compexpnew2.info, sampletype == "sample")

foo <- list(0)
results <- rep(foo,nrow(compexpnew2.sample))
for(i in 1:length(results)) { results[[i]] <- convert2prop(post, compexpnew2.sample[i,13]) }

compexpnew2.sample$propest <- unlist(lapply(results, median))
prop.hpdi <- matrix(unlist(lapply(results, HPDI, prob = 0.95)), ncol = 2, byrow = T)
compexpnew2.sample$proplow <- prop.hpdi[,1]
compexpnew2.sample$prophigh <- prop.hpdi[,2]
compexpnew2.sample$propsd <- unlist(lapply(results, sd))

### ** Make helper variables and combine the different datasets

##Dropping the starting proportions from the data
modeldata <- compexp.sample[-c(1:4),]
modeldata <- modeldata[,-2]
##Making a factor for the four different treatments
modeldata$competition <- factor(paste(modeldata$strain1, modeldata$strain2))
modeldata$comp <- as.numeric(modeldata$competition)


temp <- cbind(generation = rep(0,20), compexp.sample[c(5:14,17:26), c("replicate", "strain1", "strain2", "mat1", "mat2", "csr1", "csr2", "population")], propest = rep(0.5, 20), proplow = rep(0.01, 20), prophigh = rep(0.01, 20), propsd = rep(0.01,20))
temp$comp <- factor(paste(temp$strain1, temp$strain2))
temp$comp <- as.numeric(temp$comp)
temp2 <- compexp.sample[-c(1:4), c("generation", "replicate", "strain1", "strain2", "mat1", "mat2", "csr1", "csr2", "population", "propest", "proplow", "prophigh", "propsd")]
temp2 <- cbind(temp2, comp = modeldata$comp)
modeldata2 <- rbind(temp, temp2)
modeldata2$matdif <- ifelse(modeldata2$mat1 == modeldata2$mat2, 0, 1) #Same or different mating types?
t1 <- rep(0,80)
t1 <- t1 + ifelse(modeldata2$comp == 2, -1, 0)
t1 <- t1 + ifelse(modeldata2$comp == 3, 1, 0)
modeldata2$mdnew <- t1
modeldata2$propsd[modeldata2$propsd == 0] <- 0.01 #Set 0 prop sd's to a small value

#Set the generation zero for compexpnew.sample data
compexpnew.sample <- arrange(compexpnew.sample, generation)
#compexpnew.sample[1:4,] #This now contains initial generation 0 measurements
temp <- bind_rows(replicate(10, compexpnew.sample[1:4,], simplify = FALSE))
temp <- arrange(temp, Sample)
temp$replicate <- rep(6:15,4)
temp$population <- 21:60

compexpnew.sample <- rbind(temp, compexpnew.sample[-c(1:4),])

#Adding the second plate
compexpnew.sample <- rbind(compexpnew.sample, compexpnew2.sample)

compexpnew.sample$strain2 <- ifelse(compexpnew.sample$mat2 == 1, "csr_A", "csr_a")
compexpnew.sample$strain2 <- factor(compexpnew.sample$strain2)

compexpnew.sample$propsd[compexpnew.sample$propsd == 0] <- 0.01 #Set 0 prop sd's to a small value

#Removing extra columns from compexpnew.sample
compexpnew.sample <- compexpnew.sample[,-c(1,2,3,13,14,15)]

#Adding the final helper columns
comp <- factor(paste(compexpnew.sample$strain1, compexpnew.sample$strain2))
comp <- as.numeric(comp)
compexpnew.sample$comp <- comp
compexpnew.sample$matdif <- ifelse(compexpnew.sample$mat1 == compexpnew.sample$mat2, 0, 1) #Same or different mating types
t1 <- rep(0,80)
t1 <- t1 + ifelse(compexpnew.sample$comp == 2, -1, 0)
t1 <- t1 + ifelse(compexpnew.sample$comp == 3, 1, 0)
compexpnew.sample$mdnew <- t1

#Combining datasets
compexpall <- rbind(modeldata2, compexpnew.sample)

#Saving the combined dataset
#save(compexpall, file = "compexp_all.RData")

#Change strain levels
levels(compexpall$strain1) <- c("2489 mat A", "2489 mat a")
levels(compexpall$strain2) <- c("2489 mat A csr-1*", "2489 mat a csr-1*")


### ** Analysis of competition experiments

##Model with unique intercept for each population (now there is a mixture of starting proportions)
w.alt2 <- map2stan(
    alist(
        prop ~ dnorm(mu, sigma),
        logit(mu) <- a_comp[population] + B_csr*generation + B_pop[population]*generation + B_matA*mdnew*generation,
        propest ~ dnorm(prop, propsd),
        a_comp[population] ~ dnorm(0,0.065),
        B_csr ~ dnorm(0,1),
        B_pop[population] ~ dnorm(0,1),
        B_matA ~ dnorm(0,1),
        sigma ~ dcauchy(0,2)
        ),
    data = compexpall, start = list(prop = compexpall$propest), WAIC = F, warmup = 1000,  iter = 10000, chains = 2, cores = 2)

wpost <- extract.samples(w.alt2)

###Calculating fitnesses for w model, new matA estimation
wdata <- data.frame(exp(wpost$B_csr), exp(wpost$B_matA))
colnames(wdata) <- c("w2", "w1")
wdatalong <- gather(wdata, key = "effect", value = "s")
wdatalong$effect <- factor(wdatalong$effect)

apply(wdata, 2, median)
apply(wdata, 2, HPDI, prob = 0.95)


##Plotting a figure with proportion data and competitive fitness estimates
p1 <- ggplot(compexpall, aes(x = generation, y = propest, ymin = proplow, ymax = prophigh, group = population)) +
    geom_pointrange(fatten = 2) +
    geom_line() +
    xlab("Transfer") +
    ylab("csr-1* proportion") +   
    facet_grid(strain1 ~ strain2)

p4 <- ggplot(wdatalong, aes(x = s, y = effect)) +
    geom_halfeyeh(fill = "#66CCFF", point_interval = median_hdi) +
    geom_vline(xintercept = 1, lty = "dashed") +
    scale_x_continuous(limits = c(0,2.5), breaks = seq(0, 2.5, by = 0.5)) +
    ylab(NULL) +
    xlab(label = expression("W"[ij])) +
    scale_y_discrete(labels = c("Effect of mat A", "Effect of csr-1*")) #Note csr is w2 to get order correct here

p.final <- plot_grid(p1, p4, align = "h", axis = "b", ncol = 2, rel_widths = c(1.3,1), labels = c("A", "B"))

save_plot("compexp_final_new.pdf", p.final, base_height = 6, base_aspect_ratio=2)

### * Second competition experiment with qde-2

### ** Loading the data and calculating differences and melting curves

qde2exp.RFU <- loadHrmDataFile("190508_hrm.csv") #Load RFU data, no need to modify the file manually
qde2exp.info <- read.csv("190508_sampleinfo.csv", header = T) #Load sample information
qde2exp.Cq <- loadHrmDataFile("190508_Cq.csv")

##Define active melt region (based on first derivate curve)
##For this data, c(75, 87)
qde2exp.RFU <- filter(qde2exp.RFU, Temperature >= 75 & Temperature <= 87)

##Normalization of RFU data
## I want to get RFU from 1 to 0, subtract minimum and then divide by the new maximum
##Normalize RFU data first
qde2exp.norm <- data.frame(Temperature = qde2exp.RFU[,1], apply(qde2exp.RFU[,-1], 2, RFUnorm))

#Find melting temperatures, using spline interpolation
qde2exp.melt <- findMeltingTemp(qde2exp.norm, range = c(75, 87), step = 1e-4)

##Calculating data for a difference plot
##Normalizing for C12, which is pos. cont. for csr-tag (see lab book page 151)
qde2exp.dif <- qde2exp.norm[,-1] - qde2exp.norm$C12
qde2exp.dif <- data.frame(Temperature = qde2exp.RFU[,1], qde2exp.dif)
##Maximal difference in plot is for temperature:
qde2exp.dif[,1][which(qde2exp.dif$B11 == max(qde2exp.dif$B11))] #81.2

##Making a combined dataframe
qde2exp.info$diff <- as.numeric(qde2exp.dif[which(qde2exp.dif$B11 == max(qde2exp.dif$B11)),-1])
qde2exp.info$Tm <- qde2exp.melt
qde2exp.info$Cq <- qde2exp.Cq$Cq


#Looking at standard curve
Bstd <- filter(qde2exp.info, sampletype == "std")
Bstd <- Bstd[,c(1:3,16:18)]
#Removing samples high Cq
Bstd <- filter(Bstd, Cq < 28)

#Bstd <- filter(sample.summary, grepl("C",sample.summary$Sample))

ggplot(Bstd, aes(x = diff, y = proportion)) +
    geom_smooth(method = "lm") +
    geom_point()

### ** Fitting the standard curve

##Better to estimate standard curve using the model y = a + Bx, where x = proportion, that way we can avoid the wonky distribution of proportion
std.model <- map2stan(
    alist(
        diff ~ dnorm(mu, sigma),
        mu <- a + B*proportion,
        a ~ dnorm(0,10), #Weakly regularizing priors
        B ~ dnorm(0,10),
        sigma ~ dcauchy(0,2)
        ),
    data = Bstd, chains = 2, cores = 2, warmup = 1000, iter = 3000)

##Extract posterior samples
post <- extract.samples(std.model)

#Converted std curve
pred.dif <- matrix(seq(from = -0.05, to = 0.45, by = 0.001), nrow = 1)
convr <- apply(pred.dif, 2, convert2prop, post = post)
conv.med <- apply(convr, 2, median)
conv.int <- apply(convr, 2, HPDI, prob = 0.95)

##plotting std curve
 ggplot() +
    geom_ribbon(aes(ymin = conv.int[1,], ymax = conv.int[2,], x = as.vector(pred.dif)), fill = "grey70", alpha = 0.5) +
    #geom_ribbon(aes(ymin = conv.intpi[1,], ymax = conv.intpi[2,], x = as.vector(pred.dif)), fill = "grey70", alpha = 0.2) +    
    geom_line(aes(x = as.vector(pred.dif), y = conv.med)) +
    geom_point(aes(x = Bstd$diff, y = Bstd$proportion)) +
    xlab("Difference (norm. RFU)") +
    ylab("csr-tag proportion") +
    scale_y_continuous(limits = c(0,1), breaks = seq(0, 1, 0.1))

##
##Proportions in unknown samples
qde2exp.sample <- filter(qde2exp.info, sampletype == "sample")

foo <- list(0)
results <- rep(foo,nrow(qde2exp.sample))
for(i in 1:length(results)) { results[[i]] <- convert2prop(post, qde2exp.sample[i,16]) }

qde2exp.sample$propest <- unlist(lapply(results, median))
prop.hpdi <- matrix(unlist(lapply(results, HPDI, prob = 0.95)), ncol = 2, byrow = T)
qde2exp.sample$proplow <- prop.hpdi[,1]
qde2exp.sample$prophigh <- prop.hpdi[,2]
qde2exp.sample$propsd <- unlist(lapply(results, sd))


### ** Analysis of the qde-2 competition experiment

##Transfer 0 samples, need to be assigned replicate numbers so that lines are connected properly in ggplot
temp <- filter(qde2exp.sample, generation == 0)
temp <- rbind(temp, temp, temp, temp, temp)
temp$replicate <- sort(rep(1:5, 4))
temp2 <- filter(qde2exp.sample, generation != 0)
qde2exp.sample <- rbind(temp, temp2)
koe <- factor(qde2exp.sample$competition)
levels(koe) <- c("mat A qde-2 vs. mat a csr-1*", "mat A csr-1* qde-2 vs. mat a", "mat A vs. mat a csr-1* qde-2", "mat A csr-1* vs. mat a qde-2")
qde2exp.sample$competition <- koe
##Fix also population vector
popvec <- c(1 + 5*c(0,1,2,3), 2 + 5*c(0,1,2,3), 3 + 5*c(0,1,2,3), 4 + 5*c(0,1,2,3), 5 +5*c(0,1,2,3) )
qde2exp.sample$population[1:20] <- popvec
##Fix comp vector (numeric for competition)
qde2exp.sample$comp <- as.numeric(qde2exp.sample$competition)
##Fix propsd (this must be > 0), there is a dnorm(prop, propsd) in the model and sd cannot be 0
qde2exp.sample$propsd[qde2exp.sample$propsd == 0] <- 0.0001

##Plotting the data
#pdf("qde2comp.pdf")
p1 <- ggplot(qde2exp.sample, aes(x = generation, y = propest, ymin = proplow, ymax = prophigh, group = replicate)) +
    geom_pointrange() +
    geom_line() +
    xlab("Transfer") +
    ylab("csr-tag proportion") +
    facet_wrap( ~ competition, ncol = 2)
#dev.off()

##Alternative model, transform data such that we look at the frequency of qde-2 (irrespective of csr tag) and then estimate effects of mating type and csr-tag on qde-2 fitness.
##Data transformation
qde2new <- qde2exp.sample
tempprop <- 1 - qde2new$propest[qde2new$comp == 1 | qde2new$comp == 4]
qde2new$propest[qde2new$comp == 1 | qde2new$comp == 4] <- tempprop
colnames(qde2new)[14] <- "qde2matA"
qde2new$qde2csr <- ifelse(qde2new$comp == 2 | qde2new$comp == 3, 1, 0)
qde2new$matdif <- qde2new$qde2matA + ifelse(qde2new$qde2matA == 0, -1, 0) ## is mat A with qde-2 mutation? 1 = yes, -1 = no
qde2new$csrdif <- qde2new$qde2csr + ifelse(qde2new$qde2csr == 0, -1, 0) ## is csr-tag with qde-2 mutation? 1 = yes, -1 = no


##Model to estimate effects of qde-2, csr-tag, and mat locus but more more clever model ; )
##This is the final model
w.model3 <- map2stan(
    alist(
        prop ~ dnorm(mu, sigma),
        logit(mu) <- a_comp[comp] + B_qde2*generation + B_pop[population]*generation + B_csr*csrdif*generation + B_matA*matdif*generation,
        propest ~ dnorm(prop, propsd),
        a_comp[comp] ~ dnorm(0,0.065),
        B_pop[population] ~ dnorm(0,1),
        B_qde2 ~ dnorm(0,1),
        B_csr ~ dnorm(0,1),
        B_matA ~ dnorm(0,1),
        sigma ~ dcauchy(0,2)
        ),
    data = qde2new, start = list(prop = qde2new$propest), WAIC = F, warmup = 1000,  iter = 5000, chains = 2, cores = 2)

##Extract fitnesses
wpost3 <- extract.samples(w.model3)

##Calculating fitness for model 3
wdata3 <- data.frame(exp(wpost3$B_qde2), exp(wpost3$B_matA), exp(wpost3$B_csr))
colnames(wdata3) <- c("w1", "w2", "w3")
wdatalong3 <- gather(wdata3, key = "effect", value = "s")
wdatalong3$effect <- factor(wdatalong3$effect)

apply(wdata3, 2, median)
apply(wdata3, 2, HPDI, prob = 0.95)

p3 <-  ggplot(wdatalong3, aes(x = s, y = effect)) +
    geom_halfeyeh(fill = "#66CCFF", point_interval=median_hdi) +
    geom_vline(xintercept = 1, lty = "dashed") +
    scale_x_continuous(limits = c(0,2.5), breaks = seq(0, 2.5, by = 0.5)) +
    ylab(NULL) +
    xlab(label = expression("W"[ij])) +
    scale_y_discrete(labels = c("Effect of qde-2", "Effect of mat A", "Effect of csr-tag")) 

##Making a combined plot
p.final <- plot_grid(p1, p3, align = "h", axis = "b", ncol = 2, rel_widths = c(1.3,1), labels = c("A", "B"))

save_plot("qde2exp.pdf", p.final, base_height = 6, base_aspect_ratio=2)
